#
#########################################################
# Name: main_prg.py
# Porpose:  Boot straps and arg parser for audiomass script
# Writer: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2015/2017 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Version: (Ver.0.7) December 2017
# Rev: dec 9 2017, Aug 8 2019
#########################################################
import sys
import os
from admspack.whichcraft import check_dependencies
from admspack.datastrings import info
from admspack.process_file import file_parser
from admspack.process_dir import  dir_parser
from admspack.process_batch import batch_parser

AUTHOR, MAIL, COPYRIGHT, VERSION, RELEASE, RLS_NAME, PRG_NAME, URL, \
    SHORT_DESCRIPT, LONG_DESCRIPT, USAGE, LICENSE, SHORT_LICENSE, TRY = info()

warnings = 'audiomass: \033[33;7;3mwarning:\033[0m'
errors = 'audiomass: \033[31;7;3merror:\033[0m'
file_access = "%s Unable to access, invalid file-name  >\033[0m" % errors
dir_access = "%s Unable to access, Invalid dir-name  >\033[0m" % errors
title = ("""\033[1m%s\033[0m - audio conversion utility""" %(RLS_NAME))

def main():
    """
    Boot straps and arg parsing
    """
    if '-f' in sys.argv or '--file' in sys.argv:
        print (title)
        get_file()
    elif '-d' in sys.argv or '--dir' in sys.argv:
        print (title)
        get_dir()
    elif '-b' in sys.argv or '--batch' in sys.argv:
        print (title)
        get_batch()
    elif '-C' in sys.argv or '--check' in sys.argv:
        print (title)
        check_dependencies()
    elif '-v' in sys.argv or '--version' in sys.argv:
        print ("%s - version %s - released %s" % (RLS_NAME, VERSION, 
                                                  RELEASE))
    elif '-h' in sys.argv or '--help' in sys.argv:
        print (USAGE)
    elif '-c' in sys.argv or '--copying' in sys.argv:
        print (LICENSE)
    elif len(sys.argv) == 1:
        print (title)
        print (TRY)
    else:
        sys.exit("%s Invalid options: '%s' \n%s" % (errors,
                                            sys.argv[1:][0], TRY))
    return 0

def get_file():
    """
    Get the filename and output dirname if output dirname in sys.argv
    """
    opts = [x for x in ['-o','--output'] if x in sys.argv]
    try:
        if os.path.isfile(sys.argv[2]): # must be file only here
            path_in = os.path.abspath(sys.argv[2]) # is a file-name
        else:
            sys.exit("%s %s" % (file_access, sys.argv[2]))
    except IndexError:
        sys.exit("%s Missing argument after '%s' option.\n%s" % (errors,
                                                    sys.argv[1],TRY))
    if len(sys.argv) == 4 or len(sys.argv) >= 4:
        #if sys.argv[3] != '-o':
        if sys.argv[3] not in opts:
            sys.exit("%s Invalid option: '%s' \n%s"% (errors,
                                                      sys.argv[3], TRY))
        if len(sys.argv) >= 5:
            if os.path.isdir(sys.argv[4]):
                path_out = os.path.abspath(sys.argv[4]) # this is a dir-name
            else:
                sys.exit("%s %s"%(dir_access ,sys.argv[4]))
        else:
            sys.exit("%s Missing output dir-name after '%s' option.\n%s" % (
                                                        errors, opts[0],TRY))
    else:
        path_out = None
    # input_format is the extension format of path_in
    input_format = os.path.splitext(path_in)[1].replace(".","")
    file_parser(input_format, path_in, path_out)

def get_dir():
    """
    Get input dirname and output dirname if output dirname in sys.argv
    """
    opts = [x for x in ['-o','--output'] if x in sys.argv]
    try:
        if os.path.isdir(sys.argv[2]): 
            path_in = os.path.abspath(sys.argv[2]) # this is a dir-name
        else:
            sys.exit("%s %s"%(dir_access ,sys.argv[2]))
    except IndexError:
        sys.exit("%s Missing argument\n%s" % (errors,TRY))
    if len(sys.argv) == 4 or len(sys.argv) >= 4:
        if sys.argv[3] not in opts:
            sys.exit("%s Invalid option: '%s' \n%s"% (errors,
                                                      sys.argv[3], TRY))
        if len(sys.argv) >= 5:
            if os.path.isdir(sys.argv[4]):
                path_out = os.path.abspath(sys.argv[4]) # this is a dir-name
            else:
                sys.exit("%s %s"%(dir_access ,sys.argv[4]))
        else:
            sys.exit("%s Missing output dir-name after '%s' option.\n%s" % (
                                                        errors, opts[0],TRY))
    else:
        path_out = None
    dir_parser(path_in, path_out)

def get_batch():
    """
    Groups a queued input file stream and puts it in the queue list. 
    Call the specified function and send queue list and output dirname 
    if output dirname in sys.argv.
    """
    queue = []
    opts = [x for x in ['-o','--output'] if x in sys.argv]
    if opts:
        opt = sys.argv.index(opts[0])
        try:
            if sys.argv[opt +1]:# se trovo percorso dopo opzione -o --output
                path_O = sys.argv[opt +1]
                
        except IndexError:
            sys.exit( "%s Missing argument after '%s' option.\n%s" % (
                                                        errors, opts[0], TRY))
        else:
            if os.path.isdir(path_O):#se opzione e percorso output corretti
                arg = sys.argv[2:] # incorpora solo gli input pathnames
                arg.remove(opts[0])# infatti rimuovo '-o' , '--output'
                arg.remove(path_O) # e rimuovo l'eventuale output pathname
                path_O = os.path.abspath(path_O)#ora setto il pathname
            else:
                sys.exit("%s %s"%(dir_access ,path_O))
    else:
        path_O = None
        arg = sys.argv[2:] # dal 2° arg. lista

    for f in arg:
        if os.path.isfile(os.path.abspath(f)): # must be file only here
            queue.append(f) 
        else:
            sys.exit("%s '%s'" % (file_access, f))
    if not queue:
        sys.exit("%s Missing argument after '%s' option.\n%s" % (errors,
                                                    sys.argv[1],TRY))
    batch_parser(queue, path_O)


if __name__ == '__main__':
    status = main()
    sys.exit(status)



#
#########################################################
# Name: audio_formats.py (module)
# Porpose: module to pair audio formats
# Writer: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Version: (Ver.0.6) Febbruary 2015
# Rev
#########################################################

from admspack.comparisions import supported_formats, comparision

class Audio_Formats(object):
    """
    interface for comparison audio formats in the conversion 
    processes where different libraries and codecs are used.
    """
    def __init__(self, input_format):
        """
        Can be accept one argument of type string or None values.
        If not None, Argument is the audio input extension string 
        without dot punctuation.  
        """
        self.input_format = input_format
        self.output_format = None
        self.retcode = None # return a data tuple

    def input_selector(self, input_selection):
        """
        Accept number string argument and return the corresponding 
        format or None. This method is related to the graphic input menu
        and from process_dir only. it is useful to limit the choices within 
        the available options of the graphic input menu.
        """
        if input_selection in supported_formats():
            # mi da il formato lower case:
            self.input_format = supported_formats()[input_selection][2]
        else:
            self.input_format = None
            return self.input_format
        return self.input_format

    def output_selector(self, output_selection):
        """
        looking for a comparison/compatibilities between the input 
        format and the output format.
        Accept letters string in corresponding with output menu. see the
        output_menu in comparisions module.
        """
        if output_selection in supported_formats():
            # mi da il formato lower case:
            self.output_format = supported_formats()[output_selection][2]
        else:
            return self.output_format  # output_format = None
        return self.output_format

    def pairing_formats(self,):
        """
        returns data for that pair input_format and output_format
        """
        pair = '%s > %s' % (self.input_format, self.output_format)

        if self.retcode == 'key_error':
            self.retcode = comparision
        else:
            self.retcode = comparision(pair)
        return self.retcode

    def quality_level(self, dict_bitrate, level):
        """
        If audio codec support audio bitrate level, this method 
        define true or false and return a level of bitrate.
        """
        if level in dict_bitrate:
            valid = True
            return dict_bitrate[level]
        else:
            valid = False
            return valid



#
#########################################################
# Name: process_file.py (module)
# Porpose: parsing data files controls.
# Writer: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Version: (Ver.0.7) Nov. 2017
# Rev.
#########################################################
import subprocess
import sys
import os
from admspack.audio_formats import Audio_Formats
from admspack.comparisions import supported_formats, graphic_menu, build_cmd

warnings = 'audiomass: \033[33;7;3mWarning!\033[0m'
errors = 'audiomass: \033[31;7;3mError!\033[0m'

def file_parser(input_format, path_name, path_O):
    """
    Get the input filename format and check if input format is supported;
    If input format is supported, Dispay an output menu with the options 
    conversion based on input format.
    Also, get a tuple with specified command elements for process
    conversion.
    """
    input_selection = None
    rawmenu = graphic_menu()
    
    for supp in supported_formats().values():
        if input_format in supp[1:3]:
            input_selection = supp[0]
            menu = [rawmenu[i] for i in range(len(rawmenu)) if i not in 
                                                            set(supp[3])]
    if input_selection is None:
        # the file-name must be supported and match with dict keys
        sys.exit('\n%s File not supported: "%s"\n' % (errors, 
                                                 os.path.basename(path_name)))
    print ("\n    Available formats for "
              "encoding/decoding '\033[32;1m%s\033[0m' audio " 
              "stream" % input_format.lower()
            )
    for outformat in menu:
        print ("    %s"%(outformat))
    output_selection = input("    Type a number corresponding"
                                            " format and press enter key... ")
    main = Audio_Formats(input_format.lower())# Have a ext input >
    output_format = main.output_selector(output_selection)# get out format
    tuple_data = main.pairing_formats()# return a tuple data of the codec
    if output_selection == 'a' or output_selection == 'A':
            print('audiomass: \033[1mAbort!\033[0m')
            sys.exit()
    elif output_format is None:
        sys.exit("\n%s Unknow option '%s', Abort!" % (errors, output_selection))
    if tuple_data == 'key_error':
        sys.exit("\n%s No match available for '%s' option, Abort!" % (errors, 
                                                            output_selection))

    bitrate_test(tuple_data, output_format, path_name, path_O)

def bitrate_test(tuple_data, output_format, path_name, path_O):
    """
    Check if codec support bitrate for level compression.
    """
    dict_bitrate = tuple_data[1] # a dictionary of corresponding bitrate values
    graphic_bitrate = tuple_data[2]# a list with strings for graphic of choice
    contestual_text = tuple_data[3]# 'please, select a bitrate value'
    #output_format = tuple_data[4] # non serve, gia definito
    if dict_bitrate is None:
        bitrate = ''
    else:
        #subprocess.call(['clear'])
        print (graphic_bitrate)
        level = input(contestual_text)

        a = Audio_Formats(None)
        bitrate = a.quality_level(dict_bitrate, level)
        valid = bitrate

        if valid is False:
            print ("\n%s Unknow option '%s', "
                   "...use default\n" % (warnings, level)
                   )
            bitrate = ''

    command_builder(tuple_data, bitrate, output_format, path_name, path_O)

def command_builder(tuple_data, bitrate, output_format, path_name, path_O):
    """
    command_builder build the command, with the options, the paths names, etc.
    The 'id_codec' variable, contains the key (codec) for an corresponding 
    value with command_dict.
    """
    id_codec = tuple_data[0] # as lame --decodec or oggenc, etc
    stream_I = os.path.basename(path_name)#input, es: nome-canzone.wav'
    file_name = os.path.splitext(stream_I)[0]#only stream with no ext
    if path_O is None: # se non ce sys.argv[3]
        path_O = os.path.dirname(path_name)

    if os.path.exists('%s/%s.%s' % (path_O, file_name, output_format)):
        sys.exit("\n%s Already exists > '%s/%s.%s'" % (
                                warnings, path_O, file_name, output_format))

    command = build_cmd(id_codec, bitrate, path_name, 
                           path_O, file_name, output_format)
    print ("\n\033[36;7m %s Output Stream:\033[0m >> '%s/%s.%s'\n" 
                % (output_format, path_O, file_name, output_format))
    try:
        #print (command)# uncomment for debug
        subprocess.check_call(command, shell=True)
    except subprocess.CalledProcessError as err:
        sys.exit("audiomass:\033[31;1m ERROR!\033[0m %s" % (err))
    else:
        print ("\n\033[37;7mDone...\033[0m\n")



#
#########################################################
# Name: datastrings.py (module)
# Porpose: module for cosmetic output command line 
# Writer: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2015/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# First Release: (Ver.0.6) April 2015
# Rev: nov 22 2017, Dec 15 2017, Aug 8 2019
#########################################################

def info():################################### INFO
    """
    All general info of the audiomass-cli
    """
    AUTHOR = "Gianluca Pernigotto aka Jeanslack"
    MAIL = '<jeanlucperni@gmail.com>'
    COPYRIGHT = '© 2013-2019'
    VERSION = '0.8.0'
    RELEASE = 'Aug. 8 2019'
    RLS_NAME = "Audiomass"
    PRG_NAME = "audiomass"
    URL = "https://github.com/jeanslack/audiomass"
    SHORT_DESCRIPT = 'Audiomass is a audio conversion utility.'
    LONG_DESCRIPT = """
**Audiomass** is a command line wrapper that interfaces on differents 
audio codecs for multiple input data streams conversions. It can be easily 
implemented with other audio library codecs and currently supports: Flac, 
Lame, Vorbis-tools, Monkey's Audio, Shntool FFmpeg, etc.
"""

    SINAPSY = ("usage: audiomass [-h HELP] [-v VERSION] [-c COPYING] "
               "[-C CHECK] [-f FILE] [-d DIRECTORY] "
               "[-b BATCH [..FILE1 ..FILE2 ..FILE3 ..]] [-o DIRNAME]")
    USAGE = """%s 
  
Optional arguments:
   -f  --file     single audio stream conversion.   
   -d, --dir      specifies a directory to process   
   -b, --batch    run a process for different queued file formats   
   -o  --output   write the output streams into specified folder   
   -C, --check    Check for required dependencies   
   -h, --help     print this help and exit   
   -v, --version  print version and date and exit   
   -c, --copying  print license and exit """ % SINAPSY
  
    TRY = "type 'audiomass -h' and/or more detailed 'man audiomass'."

    LICENSE = ("""
    Copyright %s - %s
    Mail: %s

    Audiomass is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or (at 
    your option) any later version.

    Audiomass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with audiomass; if not, see <http://www.gnu.org/licenses/> 
    or write to the Free Software Foundation, Inc., 51 Franklin St, 
    Fifth Floor, Boston, MA  02110-1301 USA
    ==============================================================
    """ % (COPYRIGHT, AUTHOR, MAIL))

    SHORT_LICENSE = "'Gnu GPL3 (Gnu Public License)"

    return (AUTHOR, 
            MAIL, 
            COPYRIGHT, 
            VERSION, 
            RELEASE, 
            RLS_NAME, 
            PRG_NAME, 
            URL, 
            SHORT_DESCRIPT, 
            LONG_DESCRIPT, 
            USAGE, 
            LICENSE, 
            SHORT_LICENSE, 
            TRY
            )




import os
from shutil import which

def check_dependencies(arg=None):
    """
    Check for dependencies into your system (compatible with Linux, 
    MacOsX, Windows)
    When not arg, print a list of all required dependencies, otherwise accept
    *one* name only to pass at the function parameter and return *None* if 
    not exist or return its executable path-name.
    """
    if not arg:
        listing = ['ffmpeg', 'flac', 'lame', 'oggdec', 'oggenc', 
                   'shntool', 'mac']
        #listing = ['sox', 'wavpack']# this are for futures implementations
        for required in  listing:
            #if which(required):
            if which(required, mode=os.F_OK | os.X_OK, path=None):
                print ("Check for: '%s' ..Ok" % required)
            else:
                print ("Check for: '%s' ..Not Installed" % required)
    else:
        return which(arg, mode=os.F_OK | os.X_OK, path=None)




#
#########################################################
# Name: getcommand.py (module)
# Porpose: parsing data directory content.
# Writer: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Version: (Ver.0.7) Nov. 2017
#########################################################
import subprocess
import glob
import sys
import os
from admspack.audio_formats import Audio_Formats
from admspack.comparisions import graphic_menu, build_cmd, supported_formats

warnings = 'audiomass: \033[33;7;3mWarning!\033[0m'
errors = 'audiomass: \033[31;7;3mError!\033[0m'

def dir_parser(path_I, path_O):
    """
    1 - Show input menu for files input format choice
    2 - Creation output menu
    During output menu creation, Get a tuple with specified command
    elements
    """
    print('    Select the audio input file format in your directory:')
    for inputformat in graphic_menu():# realizzazione menu di output
                    print ("    %s"%(inputformat))

    input_selection = input("    Type a number corresponding"
                         " format and press enter key... ") # una stringa num.

    main = Audio_Formats(None) # not have a ext input = None

    input_format = main.input_selector(input_selection) # return a input format string
    if input_selection == 'a' or input_selection == 'A':
            print('audiomass: \033[1mAbort!\033[0m')
            sys.exit()
    elif input_format is None:
            sys.exit("\n%s Unknow option '%s' in select input format, Abort!" % 
                                                    (errors,input_selection))
    rawmenu = graphic_menu()
    sel = [x for x in supported_formats().values() if input_format in x[1:3]]
    menu = [rawmenu[i] for i in range(len(rawmenu)) if i not in set(sel[0][3])]
    for outformat in menu:
        print ("    %s"%(outformat))
    output_selection = input("    Type a number corresponding"
                                            " format and press enter key... ")
    output_format = main.output_selector(output_selection)
    tuple_data = main.pairing_formats()# return a tuple data of the codec

    if output_selection == 'a' or output_selection == 'A':
            print('audiomass: \033[1mAbort!\033[0m')
            sys.exit()
    elif output_format is None:
        sys.exit("\n%s Unknow option '%s' in select output format, Abort!" % 
                                                    (errors,output_selection))
    if tuple_data == 'key_error':
        sys.exit("\n%s No match available for '%s' option, Abort!" % (errors, 
                                                            output_selection))

    bitrate_test(tuple_data, output_format, path_I, path_O, input_format)

def bitrate_test(tuple_data, output_format, path_I, path_O, input_format):
    """
    Check if codec support bitrate for level compression.
    
    """
    dict_bitrate = tuple_data[1] # a dictionary of corresponding bitrate values
    graphic_bitrate = tuple_data[2]# a list with strings for graphic of choice
    contestual_text = tuple_data[3]# 'please, select a bitrate value'
    #output_format = tuple_data[4] # non serve, gia definito
    if dict_bitrate is None:
        bitrate = ''
    else:
        print (graphic_bitrate)
        level = input('%s' % contestual_text)

        a = Audio_Formats(None)
        bitrate = a.quality_level(dict_bitrate, level)
        valid = bitrate

        if valid is False:
            print ("\n%s Unknow option '%s', ...use default\n" % (warnings, 
                                                                  level))
            bitrate = ''
            
    command_builder(tuple_data, bitrate, output_format, path_I, 
                                        path_O, input_format)

def command_builder(tuple_data, bitrate, output_format, path_I, 
                                        path_O, input_format):
    """
    command_builder is based on construction of the paths and formats
    strings (output_format, path_in, path_O) and the 'command' variable, that 
    contains the key (codec) for an corresponding values used for process.
    
    WARNING : path_I e path_O devono rappresentare solo il nome dei percorsi
    file_name = 'only stream with no ext, es: nome_canzone'
    path_name = 'complete path input: /dir/dir/filename.ext'
    stream_I = 'Nome dello stream di input, es: nome-canzone.wav'
    stream_O = 'Nome dello stream di output, es: nome-canzone.flac'
    path_I = '/home/Musica solo input dir-name'
    path_O = '/dir/dir solo output dir-name'
    """
    id_codec = tuple_data[0] # as lame --decodec or oggenc, etc
    exe = 'False'
    count = 0
    for upper_lower_name in [input_format, input_format.upper()]:
        # itero su nomi formato upper-case e lower-case
        for path_name in glob.glob("%s/*.%s" % (path_I, upper_lower_name)):
            stream_I = os.path.basename(path_name)
            file_name = os.path.splitext(stream_I)[0]
            exe = None
            count += 1
            if path_O is None: # se non ce sys.argv[3]
                path_O = os.path.dirname(path_name)
            if os.path.exists('%s/%s.%s' % (path_O, file_name, output_format)):
                print ("\n%s Already exists > '%s/%s.%s' >> skipping >>" % (
                        warnings, path_O, file_name, output_format)
                        )
                continue
            
            command = build_cmd(id_codec, bitrate, path_name, 
                            path_O, file_name, output_format)
            print ("\n\033[36;7m|%s| %s Output Stream:\033[0m >> '%s/%s.%s'\n" 
                    % (str(count),output_format, path_O, file_name, 
                       output_format))
            try:
                #print (command) # uncomment for debug
                subprocess.check_call(command, shell=True)
                    
            except subprocess.CalledProcessError as err:
                sys.exit("audiomass:\033[31;1m ERROR!\033[0m %s" % (err))
        
    if exe == 'False':
            sys.exit("\n%s Files missing: No files '%s' "
                "in '%s' \n" % (errors, input_format, path_I)
                        )
    print ("\n\033[37;7mDone...\033[0m\n")





#
#########################################################
# Name: getcommand.py (module)
# Porpose: parsing the data files list for batch process.
# Writer: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Version: (Ver.0.7) Nov 27 2017, Dec.15 2017
# Rev
#########################################################
import subprocess
import sys
import os
from collections import Counter
from admspack.audio_formats import Audio_Formats
from admspack.comparisions import supported_formats, graphic_menu, build_cmd

warnings = 'audiomass: \033[33;7;3mWarning!\033[0m'
errors = 'audiomass: \033[31;7;3mError!\033[0m'

def batch_parser(f_list, path_O):
    """
    Clean-up list contaminated by duplicate files.
    """
    for k,v in Counter(f_list).items():# controllo doppioni accidentali.
        if v>1:
            print ("%s Removing following duplicates: > '%s' >" % (warnings, k))
    f_list = list(set(f_list)) # removal of any duplicates in the list
    sorting_dictionary(f_list, path_O)

def sorting_dictionary(f_list, path_O):
    """
    Dictionary creation key = format: value = [filename, filename, etc] 
    in order to group the streams with the same format into separate lists.
    """
    formats = {}#Dizionario {formato1:[file1,file2,etc],..]}
    new_list = []#La lista ripulita dai file non supportati
    all_formats = []# tutti i formati supportati 
    for selection, upper_f, lower_f, limits in supported_formats().values():
        all_formats.append(upper_f)# WAV,AIF,FLAC,APE etc
        all_formats.append(lower_f)# wav,aif,flac,ape etc

    #Creazione keys (chiavi) nel diz. formats:
    for i in f_list: #Append separated file format
        name, ext = os.path.splitext(i)
        new = ext.replace(".","")#Tolgo il punto all'estensione
        if new in all_formats:#Se estens. input è supportata
            new_list.append(i)#Lista ripulita file importati
            if new.lower() not in formats:#Add not present key at dict
                formats[new.lower()] = []#Aggiungo chiavi non presenti nel diz +
                                         #il valore lista vuota: {formato:[]}
        else:
            print ('\n%s Not supported file format: "%s%s" >> skipping >>'% (
                                                    warnings,name,ext))
    if new_list == []: # se i file importati sono tutti incompatibili
        sys.exit('\n%s ...No audio streams to process, exit!' % errors)
    # Creazione dei records (valori) nel diz. formats
    for i in new_list:
        name, ext = os.path.splitext(i)
        new = ext.replace(".","")# tolgo il punto all'estensione
        formats[new.lower()].append("%s%s" %(name,ext))
    menu_selections(formats, path_O)

def menu_selections(formats, path_O):
    """
    Creation input menu
    Creation output menu
    During output menu creation, Get a tuple with specified command
    elements
    """
    input_selection = []# deve contenere solo interi(int)
    for input_format in list(formats.keys()):#Itero sui formati importati
        # NOTE 1 RELOAD: ricarico nuovamente il grafico dei formati 
        # integralmente con l'originale 
        menu = graphic_menu()
        print ("\n    Available formats for encoding/decoding "
            "'\033[32;1m%s\033[0m' audio streams" % input_format)
        # Dizion. = {chiavi'srtinga 1':(integear,'formato')} 
        # itero sulla tupla valori 
        for v in supported_formats().values():
            if formats == {}:# se è completamente vuoto, esco
                sys.exit('\n%s...End selection process, exit!\n'% errors)
            if input_format in v[1:3]:# mi prendo gli interi corrispondenti
                input_selection.append(v[0])# v[0] mi da l'intero
                menu = [menu[i] for i in range(len(menu)) if i not in set(v[3])]
                for outformat in menu:# realizzazione menu di output
                    print ("    %s" % outformat)
                output_selection = input("    Type a number corresponding"
                                            " format and press enter key... ")
                ##------------------- CREAZIONE OUTPUT MENU ------------------#
                main = Audio_Formats(input_format)# Have a ext input >
                output_format = main.output_selector(output_selection)
                tuple_data = main.pairing_formats()# return tuple data of codec
                if output_selection == 'a' or output_selection == 'A':
                    print('audiomass: \033[1mAbort!\033[0m')
                    sys.exit()
                elif output_format is None:
                    # Se nessuna selezione e premi enter rimuovo chiave e valore 
                    # dal dizionario, cioè escludo quei files dalla conversione.
                    print ("\n%s Unknow option '%s' >> skipping" % (warnings, 
                                                             output_selection))
                    formats.pop(input_format, None)
                    continue # meglio partire da capo 

                if tuple_data == 'key_error':
                    print ("\n%s No match available for '%s' option >> "
                                 "skipping >>" % (warnings, output_selection))
                    formats.pop(input_format, None)
                    continue # troppi errori, meglio contimuare da capo

                bitrate_test(tuple_data, output_format, 
                             formats.get(input_format), path_O)

def bitrate_test(tuple_data, output_format, path_in, path_O):
    """
    Check if codec support bitrate for level compression.
    """
    dict_bitrate = tuple_data[1] # a dictionary of corresponding bitrate values
    graphic_bitrate = tuple_data[2]# a list with strings for graphic of choice
    contestual_text = tuple_data[3]# 'please, select a bitrate value'
    #output_format = tuple_data[4] # non serve, gia definito

    if dict_bitrate is None:
        bitrate = ''
    else:
        print (graphic_bitrate)
        level = input(contestual_text)
        a = Audio_Formats(None)
        bitrate = a.quality_level(dict_bitrate, level)
        valid = bitrate
        if valid is False:
            print ("\n%s Unknow quality level '%s', ...use default\n" % (
                                                            warnings, level))
            bitrate = ''
    command_builder(tuple_data, bitrate, output_format, path_in, path_O)

def command_builder(tuple_data, bitrate, output_format, path_in, path_O):
    """
    command_builder build the command, with the options, the paths names, etc.
    The 'id_codec' variable, contains the key (codec) for an corresponding 
    value with command_dict.
    """
    id_codec = tuple_data[0] # as lame --decodec or oggenc, etc
    count = 0
    not_processed = []
    for path_name in path_in:
        stream_I = os.path.basename(path_name)#input, es: nome-canzone.wav'
        file_name = os.path.splitext(stream_I)[0]#only stream with no ext
        count += 1
        if path_O is None: # se non ce sys.argv[3]
            path_O = os.path.dirname(path_name)
        if os.path.exists('%s/%s.%s' % (path_O, file_name, output_format)):
            print ("\n%s Already exists > '%s/%s.%s' >> skipping >>" % (
                    warnings, path_O, file_name, output_format)
                    )
            #path_in.remove(path_name)
            not_processed.append(path_name)
            continue
        
        command = build_cmd(id_codec, bitrate, path_name, 
                           path_O, file_name, output_format)
        print ("\n\033[36;7m|%s| %s Output Stream:\033[0m >> '%s/%s.%s'\n" 
            % (str(count),output_format, path_O, file_name, output_format))
        try:
            #print (command) # uncomment for debug
            subprocess.check_call(command, shell=True)
        except subprocess.CalledProcessError as err:
            sys.exit("audioamass:\033[31;1mERROR!\033[0m %s" % (err))
            
    if not_processed:
        for fiLe in not_processed:
            if fiLe in path_in:
                path_in.remove(fiLe)
        #print ("\n\033[31;7mStreams NOT Processed:\033[0m >> %s\n" % (not_processed))
        print ("\n\033[33;7;3mStreams NOT Processed:\033[0m")
        for list1 in not_processed:
            print(list1)
        
    #print ("\n\033[32;7mQueue Streams Processed:\033[0m >> %s\n" % (path_in))
    if path_in:
        print ("\n\033[32;7mQueue Streams Processed:\033[0m")
        for list2 in path_in:
            print(list2)
    print ("\n\033[37;7mDone...\033[0m\n")
        



#
#########################################################
# Name: comparisions.py (module)
# Porpose:  module for output strings commands and formats comparisions
# Writer: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2015/2016 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Version: (Ver.0.6) Febbruary 2015
# Rev
#########################################################

def graphic_menu():
    """
    Menu for output audio selection format used from all process(f,d,b)
    """
    return [
    "------------",
    " \033[1m 1 \033[0m> Wav   (WAVEform audio format, PCM uncompresed)", 
    " \033[1m 2 \033[0m> Aiff  (Apple Interchange File Format)", 
    " \033[1m 3 \033[0m> Flac  (Free Lossless Audio Codecs)", 
    " \033[1m 4 \033[0m> Ape   (Monkey's audio)", 
    " \033[1m 5 \033[0m> Mp3   (MPEG-1 Audio Layer 3)", 
    " \033[1m 6 \033[0m> Ogg   (ogg-vorbis lossy format)",  
    "------------", 
    " \033[41;37;1m A \033[0m \033[1m..ABORT\033[0m", 
    "------------"]

def supported_formats():
    """
    supported_format: it is useful to limit the choices within 
    the available options of the graphic_menu.
    """
    return {'1': (1,'WAV','wav',[1]), '2': (2,'AIFF','aiff',[2,4]), 
          '3': (3,'FLAC','flac',[3]), '4': (4,'APE','ape',[3,4]), 
          '5': (5,'MP3','mp3',[3,4,5,6]),'6': (6,'OGG','ogg',[3,4,5,6])
            }

def comparision(pair):
  """
  returns the required values for each pair of audio formats. 
  Accept one only string argument in this form: 'exemple > exemple'.
  You can add or remove new formats modules from here.
  """

  ####------------------------------  BITRATES MENU:
  flac_menu = ("\n\033[1m"
              "References on quality for Flac format:\033[0m\n"
              "  0 = low compression > more access space > better quality\n"
              "  1\n"
              "  2\n"
              "  3\n"
              "  4 = Great quality/compression ratio\n"
              "  5 = Is the default compression \n"
              "  6\n"
              "  7\n"
              "  8 = maximum compression > less space > lower quality\n"
              )

  ape_menu = ("\n\033[1m"
              "References on quality for Ape format:\033[0m\n"
              "  1    -c1000 = Fast (Best quality)\n"
              "  2    -c2000 = Normal\n"
              "  3    -c3000 = High\n"
              "  4    -c4000 = Extra\n"
              "  5    -c5000 = Insane\033[0m\n"
              )

  mp3_menu = ("\n\033[1m"
              "References on quality and bit-rate for MP3:\033[0m\n"
              "  0     medium    >  VBR 92 kbit\n"
              "  1     standard  >  VBR 112 kbit/s\n"
              "  2     extreme   >  VBR 150 kbit/s\n"
              "  3     insane    >  CBR 320 kbit/s\n\n"
              )

  ffmpeg_mp3_menu = ("\n\033[1m"
              "References on quality and bit-rate for MP3:\033[0m\n"
              "  0     >  VBR 128 kbit\n"
              "  1     >  VBR 160 kbit/s\n"
              "  2     >  VBR 192 kbit/s\n"
              "  3     >  VBR 260 kbit/s\n"
              "  4     >  CBR 320 kbit/s\n"
              )

  ogg_menu = ("\n\033[1m"
              "References on the quality and bit-rate for OGG:\033[0m\n"
              "  1    >   80 kbit        |     6   >   192 kbit/s\n"
              "  2    >   92 kbit/s      |     7   >   200 kbit/s\n"
              "  3    >   100 kbit/s     |     8   >   260 kbit/s\n"
              "  4    >   128 kbit/s     |     9   >   320 kbit/s\n"
              "  5    >   134 kbit/s     |     10  >   520 kbit/s\n"
              )

  ffmpeg_ogg_menu = ("\n\033[1m"
              "References on the quality and bit-rate for OGG:\033[0m\n"
              "  0    >  VBR 128 kbit\n"
              "  1    >  VBR 160 kbit/s\n"
              "  2    >  VBR 192 kbit/s\n"
              "  3    >  VBR 260 kbit/s\n"
              "  4    >  CBR 320 kbit/s\n"
              )

  #### ----------------------------- BITRATES LEVELS:

  flac_opt_comp = {"0":"-0", "1":"-1", "2":"-2", "3":"-3",
                   "4":"-4", "5":"-5", "6":"-6", "7":"-7",
                   "8":"-8"
                    }

  ape_opt_comp = {"1":"-c1000", "2":"-c2000", "3":"-c3000", 
                  "4":"-c4000", "5":"-c5000"
              }

  mp3_opt_comp = {"0":"--preset medium", "1":"--preset standard", 
                  "2":"--preset extreme", "3":"--preset insane"}

  ffmpeg_mp3_opt_comp = {"0":"-acodec libmp3lame -b:a 128k -ar 44100",  
                         "1":"-acodec libmp3lame -b:a 160k -ar 44100",
                         "2":"-acodec libmp3lame -b:a 192k -ar 44100", 
                         "3":"-acodec libmp3lame -b:a 260k -ar 44100",
                         "4":"-acodec libmp3lame -b:a 320k -ar 44100",
                          }

  ogg_opt_comp = {"1":"-q 1", "2":"-q 2", "3":"-q 3", "4":"-q 4", "5":"-q 5",
                  "6":"-q 6", "7":"-q 7", "8":"-q 8", "9":"-q 9", "10":"-q 10"
                  }

  ffmpeg_ogg_opt_comp = {"0":"-vn -acodec libvorbis -ar 44100 -ab 128k",  
                         "1":"-vn -acodec libvorbis -ar 44100 -ab 160k", 
                         "2":"-vn -acodec libvorbis -ar 44100 -ab 192k", 
                         "3":"-vn -acodec libvorbis -ar 44100 -ab 260k",  
                         "4":"-vn -acodec libvorbis -ar 44100 -ab 320k",
                            }
  ####-------------------------------  PAIRING ASSOCIATIONS FORMATS:
  object_assignment = {
      'wav > aiff' : ('shntool', None, None, None, 'aiff'), 
      'aiff > wav' : ('shntool', None, None, None, 'wav'), 
      #'aiff > ape' : ('shntool', None, None, None, 'ape'),
      'flac > wav' : ('shntool', None, None, None, 'wav'),
      'flac > aiff' : ('shntool', None, None, None, 'aiff'),
      'flac > ape' : ('shntool', None, None, None, 'ape'), 
      'ape > wav' : ('shntool', None, None, None, 'wav'),
      'ape > aiff' : ('shntool', None, None, None, 'aiff'),
      #'ape > flac' : ('shntool', None, None, None, 'flac'),

      'wav > flac' : ('flac', flac_opt_comp, flac_menu, 'Type the '
                      'compression level in digits 0 to 8, and press '
                      'enter key > ', 'flac'), 
      'aiff > flac' : ('flac', flac_opt_comp, flac_menu, 'Type the '
                      'compression level in digits 0 to 8, and press '
                      'enter key > ', 'flac'),

      'wav > mp3' : ('lame', mp3_opt_comp, mp3_menu, 'Type the compression '
                      'level in digits 0 to 3, and press enter key > ', 'mp3'),
      'aiff > mp3' : ('lame', mp3_opt_comp, mp3_menu, 'Type the compression '
                      'level in digits 0 to 3, and press enter key > ', 'mp3'),
      'mp3 > wav' : ('lame --decode', None, None, None, 'wav'),
      
      'wav > ogg' : ('oggenc', ogg_opt_comp, ogg_menu, 'Type the compression '
                      'level in digits 1 to 10, and press enter key > ', 'ogg'),
      'aiff > ogg' : ('oggenc', ogg_opt_comp, ogg_menu, 'Type the compression '
                      'level in digits 1 to 10, and press enter key > ', 'ogg'),
      'flac > ogg' : ('oggenc', ogg_opt_comp, ogg_menu, 'Type the compression '
                      'level in digits 1 to 10, and press enter key > ', 'ogg'),

      'ogg > wav' : ('oggdec', None, None, None, 'wav'),

      'wav > ape' : ('mac', ape_opt_comp, ape_menu, 'Type the compression '
                      'level in digits 1 to 5, and press enter key > ', 'ape'),

      'flac > mp3' : ('ffmpeg', ffmpeg_mp3_opt_comp, ffmpeg_mp3_menu, 
                      'Type the compression level in digits 0 to 4, '
                      'and press enter key > ', 'mp3'),
      'ape > mp3' : ('ffmpeg', ffmpeg_mp3_opt_comp, ffmpeg_mp3_menu, 
                      'Type the compression level in digits 0 to 4, '
                      'and press enter key > ', 'mp3'),
      'ape > ogg' : ('ffmpeg', ffmpeg_ogg_opt_comp, ffmpeg_ogg_menu,
                      'Type the compression level in digits 0 to 4, '
                      'and press enter key > ', 'ogg'),
      'mp3 > aiff' : ('ffmpeg', None, None, None, 'aiff'),
      'ogg > aiff' : ('ffmpeg', None, None, None, 'aiff'),
                          }
  return object_assignment.get(pair, 'key_error')

def build_cmd(id_codec, bitrate, path_name, path_O, file_name, output_format):
    """
    Each command associated with a type of codec appears to be different. 
    This problem is solved here. Return a string with the command correctly 
    formed.
    NOTE: The key of this dictionary must match with the first value of 
    the 'object assignment' dictionary to work well
    """
    command_dict = {
'flac':'flac -V %s "%s" -o "%s/%s.%s"' % (bitrate, path_name, path_O,
                                        file_name, output_format),
'lame':'lame --nohist %s "%s" "%s/%s.%s"' % (bitrate, path_name, path_O,
                                file_name, output_format),
'lame --decode':'lame --decode "%s" "%s/%s.%s"' % (path_name, path_O,
                                file_name, output_format),
'oggenc':'oggenc %s "%s" -o "%s/%s.%s"' % (bitrate, path_name, path_O,
                                        file_name, output_format),
'mac':'mac "%s" "%s/%s.%s" %s' % (path_name, path_O, file_name, output_format,
                                bitrate),
'ffmpeg':'ffmpeg -i "%s" %s "%s/%s.%s"' % (path_name, bitrate, path_O,
                                        file_name, output_format),
'oggdec':'oggdec "%s" -o "%s/%s.%s"' % (path_name, path_O, file_name,
                                    output_format),
'shntool':'shntool conv -o %s -O always "%s" -d "%s"' % (output_format, 
                                                         path_name, path_O),
                        }

    return command_dict[id_codec]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#########################################################
# Name: setup.py
# Porpose: script for building and setup audiomass package
# Platform: Gnu/Linux, Unix
# Writer: jeanslack <jeanlucperni@gmail.com>
# Copyright: (c) 2015 jeanslack <jeanlucperni@gmail.com>
# license: GPL3
# Rev: nov 22 2017, aug 8 2019
#########################################################

from distutils.core import setup
from setuptools import setup, find_packages
from admspack.datastrings import info

cr = info()
AUTHOR = cr[0]
MAIL = cr[1]
VERSION = cr[3]
RLS_NAME = cr[5]# release name first letter is Uppercase
PRG_NAME = cr[6]
URL = cr[7]
SHORT_DESCRIPTION = cr[8]
LONG_DESCRIPTION = cr[9]
SHORT_LICENSE = cr[12]# short_license

# ---- categorize with ----#
CLASSIFIERS = [
            'Development Status :: 4 - Beta',
            'Environment :: Console',
            'Intended Audience :: End Users/Desktop',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Natural Language :: English',
            'Operating System :: MacOS :: MacOS X',
            'Operating System :: POSIX',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Topic :: Multimedia :: Sound/Audio :: Conversion',
            'Topic :: Utilities',
                ]

#----------  Source/Build distributions  ----------------#

# get the package data
DATA_FILES = [('share/man/man1', ['man/audiomass.1.gz']),
              ('share/audiomass', ['AUTHORS', 'BUGS', 'CHANGELOG', 
               'COPYING', 'README.md', 'TODO']),
              ]
# Setup
setup(name=PRG_NAME,
        version=VERSION,
        description=SHORT_DESCRIPTION,
        long_description=open('README.md').read(),
        long_description_content_type='text/markdown',
        author=AUTHOR,
        author_email=MAIL,
        url=URL,
        license=SHORT_LICENSE,
        platforms=["Linux","Unix","MacOS"],
        packages=find_packages(),
        scripts=['audiomass'],
        data_files=DATA_FILES,
        classifiers=CLASSIFIERS,
        #install_requires=REQUIRES,
        )
